@@ -77,7 +77,8 @@ struct _global_handles {
     Atom tray_opcode;      /* Atom: _NET_SYSTEM_TRAY_MESSAGE_OPCODE */
     Atom xembed_info;      /* Atom: _XEMBED_INFO */
     Atom utf8_string_atom; /* Atom: UTF8_STRING */
-    Atom wm_state;         /* Atom: _NET_WM_STATE */
+    Atom wm_state;         /* Atom: WM_STATE */
+    Atom net_wm_state;     /* Atom: _NET_WM_STATE */
     Atom wm_state_fullscreen; /* Atom: _NET_WM_STATE_FULLSCREEN */
     Atom wm_state_demands_attention; /* Atom: _NET_WM_STATE_DEMANDS_ATTENTION */
     Atom wm_take_focus;    /* Atom: WM_TAKE_FOCUS */
@@ -112,7 +113,14 @@ struct genlist *embeder_list;
 typedef struct _global_handles Ghandles;
 Ghandles *ghandles_for_vchan_reinitialize;
 
-#define SKIP_NONMANAGED_WINDOW if (!list_lookup(windows_list, window)) return
+#define SKIP_NONMANAGED_WINDOW do {                                    \
+    if (!list_lookup(windows_list, window)) {                          \
+        if (g->log_level > 0)                                          \
+            fprintf(stderr, "Skipping unmanaged window 0x%x",          \
+                    (int) window);                                     \
+        return;                                                        \
+    }                                                                  \
+} while (0)
 
 void send_wmname(Ghandles * g, XID window);
 void send_wmnormalhints(Ghandles * g, XID window, int ignore_fail);
@@ -574,7 +582,7 @@ void send_window_state(Ghandles * g, XID window)
     struct msg_window_flags flags;
 
     /* FIXME: only first 10 elements are parsed */
-    ret = XGetWindowProperty(g->display, window, g->wm_state, 0, 10,
+    ret = XGetWindowProperty(g->display, window, g->net_wm_state, 0, 10,
             False, XA_ATOM, &act_type, &act_fmt, &nitems, &bytesleft, (unsigned char**)&state_list);
     if (ret != Success)
         return;
@@ -593,6 +601,7 @@ void send_window_state(Ghandles * g, XID window)
 void process_xevent_map(Ghandles * g, XID window)
 {
     XWindowAttributes attr;
+    long new_wm_state[2];
     struct msg_hdr hdr;
     struct msg_map_info map_info;
     Window transient;
@@ -616,6 +625,13 @@ void process_xevent_map(Ghandles * g, XID window)
     write_message(g->vchan, hdr, map_info);
     send_wmname(g, window);
     //      process_xevent_damage(g, window, 0, 0, attr.width, attr.height);
+
+    if (!attr.override_redirect) {
+        /* WM_STATE is always set to normal */
+        new_wm_state[0] = NormalState; /* state */
+        new_wm_state[1] = None;        /* icon */
+        XChangeProperty(g->display, window, g->wm_state, g->wm_state, 32, PropModeReplace, (unsigned char *)new_wm_state, 2);
+    }
 }
 
 void process_xevent_unmap(Ghandles * g, XID window)
@@ -630,6 +646,7 @@ void process_xevent_unmap(Ghandles * g, XID window)
     hdr.untrusted_len = 0;
     write_struct(g->vchan, hdr);
     XDeleteProperty(g->display, window, g->wm_state);
+    XDeleteProperty(g->display, window, g->net_wm_state);
 }
 
 void process_xevent_destroy(Ghandles * g, XID window)
@@ -817,8 +834,6 @@ void process_xevent_selection_req(Ghandles * g,
     Atom Targets = XInternAtom(g->display, "TARGETS", False);
     Atom Compound_text =
         XInternAtom(g->display, "COMPOUND_TEXT", False);
-    Atom Utf8_string_atom =
-        XInternAtom(g->display, "UTF8_STRING", False);
     int convert_style = XConverterNotFound;
 
     if (g->log_level > 0)
@@ -826,7 +841,7 @@ void process_xevent_selection_req(Ghandles * g,
                 XGetAtomName(g->display, req->target));
     resp.property = None;
     if (req->target == Targets) {
-        Atom tmp[4] = { XA_STRING, Targets, Utf8_string_atom,
+        Atom tmp[4] = { XA_STRING, Targets, g->utf8_string_atom,
             Compound_text
         };
         XChangeProperty(g->display, req->requestor, req->property,
@@ -839,13 +854,20 @@ void process_xevent_selection_req(Ghandles * g,
         convert_style = XTextStyle;
     else if (req->target == Compound_text)
         convert_style = XCompoundTextStyle;
-    else if (req->target == Utf8_string_atom)
+    else if (req->target == g->utf8_string_atom)
         convert_style = XUTF8StringStyle;
     if (convert_style != XConverterNotFound) {
         XTextProperty ct;
-        char *ptr[] = { NULL };
+        char empty_string[1] = { '\0' };
+        char *ptr[] = { empty_string };
         // Workaround for an Xlib bug: Xutf8TextListToTextProperty mangles
-        // certain characters.
+        // certain characters, so we check for UTF-8 validity ourselves and use
+        // XStringListToTextProperty.  If we fail a UTF-8 validity check, ptr[0]
+        // is left pointing to an empty string, which is safe.
+        //
+        // We don’t return immediately because users might (reasonably)
+        // assume that any sensitive data previously on the clipboard
+        // (such as passwords) has been overwritten.
         if (convert_style == XUTF8StringStyle &&
             !is_valid_clipboard_string_from_vm((unsigned char *)g->clipboard_data))
             fputs("Invalid clipboard data from VM\n", stderr);
@@ -1040,7 +1062,7 @@ void process_xevent_message(Ghandles * g, XClientMessageEvent * ev)
                 fprintf(stderr, "unhandled tray opcode: %ld\n",
                         ev->data.l[1]);
         }
-    } else if (ev->message_type == g->wm_state) {
+    } else if (ev->message_type == g->net_wm_state) {
         struct msg_hdr hdr;
         struct msg_window_flags msg;
 
@@ -1339,7 +1361,8 @@ void mkghandles(Ghandles * g)
     g->tray_opcode =
         XInternAtom(g->display, "_NET_SYSTEM_TRAY_OPCODE", False);
     g->xembed_info = XInternAtom(g->display, "_XEMBED_INFO", False);
-    g->wm_state = XInternAtom(g->display, "_NET_WM_STATE", False);
+    g->wm_state = XInternAtom(g->display, "WM_STATE", False);
+    g->net_wm_state = XInternAtom(g->display, "_NET_WM_STATE", False);
     g->wm_state_fullscreen = XInternAtom(g->display, "_NET_WM_STATE_FULLSCREEN", False);
     g->wm_state_demands_attention = XInternAtom(g->display, "_NET_WM_STATE_DEMANDS_ATTENTION", False);
     g->wm_take_focus = XInternAtom(g->display, "WM_TAKE_FOCUS", False);
@@ -1663,9 +1686,14 @@ void handle_focus(Ghandles * g, XID winid)
     } else if (key.type == FocusOut
             && (key.mode == NotifyNormal
                 || key.mode == NotifyUngrab)) {
-
-        XSetInputFocus(g->display, None, RevertToParent,
-                CurrentTime);
+        if ( (l=list_lookup(windows_list, winid)) && (l->data) )
+            input_hint = ((struct window_data*)l->data)->input_hint;
+        else {
+            fprintf(stderr, "WARNING handle_focus: Window 0x%x data not initialized", (int)winid);
+            input_hint = True;
+        }
+        if (input_hint)
+            XSetInputFocus(g->display, None, RevertToParent, CurrentTime);
 
         if (g->log_level > 1)
             fprintf(stderr, "0x%x lost focus\n", (int) winid);
@@ -1867,7 +1895,7 @@ void handle_window_flags(Ghandles *g, XID winid)
     read_data(g->vchan, (char *) &msg_flags, sizeof(msg_flags));
 
     /* FIXME: only first 10 elements are parsed */
-    ret = XGetWindowProperty(g->display, winid, g->wm_state, 0, 10,
+    ret = XGetWindowProperty(g->display, winid, g->net_wm_state, 0, 10,
             False, XA_ATOM, &act_type, &act_fmt, &nitems, &bytesleft, (unsigned char**)&state_list);
     if (ret != Success)
         return;
@@ -1900,7 +1928,7 @@ void handle_window_flags(Ghandles *g, XID winid)
     if (!changed)
         return;
 
-    XChangeProperty(g->display, winid, g->wm_state, XA_ATOM, 32, PropModeReplace, (unsigned char *)new_state_list, j);
+    XChangeProperty(g->display, winid, g->net_wm_state, XA_ATOM, 32, PropModeReplace, (unsigned char *)new_state_list, j);
 }
 
 void handle_message(Ghandles * g)
